#include <octave/oct.h>
#include <octave/byte-swap.h>

#include "libdld_common.h"
#include "fifo_host.h"

static bool mpi_bcast_root(const octave_value_list &args,
			   octave_value_list &retval){

  int size;
  int key ;
  int root;
  int ope_type = 0;
  
  size=args(1).int_value();
  root=args(2).int_value();
  key =args(3).int_value();

  std::basic_ostringstream<char> send_buffer;

  // Save the byte format to the stream
  oct_mach_info::float_format flt_fmt =
    oct_mach_info::native_float_format();
  
  send_buffer.write(X_CAST(char *,&flt_fmt),
                    sizeof(oct_mach_info::float_format));
  
  // Serialize all each entry in binary form
  for (int i = 0; i < 1; i++)
    {
      // Make sure the value exists
      if (args(i).is_defined())
        {
          // Convert octave_value to binary
          mpi_save_binary_data(send_buffer,
                               args(i), "V", "",
                               false, false);
        }
      else
        {
          error("mpi_send: Doesn't handle that form of data yet");

          return false;
        }
    }

  send_buffer.eof();

  std::basic_string<char> send_data = send_buffer.str();
  int buffer_length = send_data.length();
  const char *buffer_data = send_data.data();

  try
    {
      fifo_host *mpiproc=(fifo_host *)poctave_common::mpiproc;

      mpiproc->send("mpi_bcast",sizeof("mpi_bcast"));
      mpiproc->send(&ope_type,sizeof(int));

      mpiproc->send(&root,sizeof(int));

      mpiproc->send(&size,sizeof(int));

      mpiproc->send(&buffer_length,sizeof(int));
      mpiproc->send(buffer_data,buffer_length);

      mpiproc->send(&key,sizeof(int));

      int mpi_retval;
      mpiproc->recv(&mpi_retval,sizeof(int));
    }
  catch(...)
    {
      kill(poctave_common::mpi_pid,SIGURG);
      return false;
    }

  return true;

};

static bool mpi_bcast_non_root(const octave_value_list &args,
			       octave_value_list &retr_list){

  int size;
  int key ;
  int root;
  int ope_type = -1;

  size=args(0).int_value();
  root=args(1).int_value();
  key =args(2).int_value();

  fifo_host *mpiproc=(fifo_host *)poctave_common::mpiproc;

  try
    {
      mpiproc->send("mpi_bcast",sizeof("mpi_bcast"));
      mpiproc->send(&ope_type,sizeof(int));
      
      mpiproc->send(&root,sizeof(int));
      
      mpiproc->send(&size,sizeof(int));
      mpiproc->send(&key,sizeof(int));
      
      int buffer_length;
      mpiproc->recv(&buffer_length,sizeof(int));
      char buffer_data[buffer_length];
      mpiproc->recv(buffer_data,buffer_length);

      // Pull the entry _start and _length data off the end of the buffer
      std::basic_stringstream<char> recv_buffer;
      std::basic_streambuf<char> *rdbuf;

      rdbuf = recv_buffer.rdbuf();
      rdbuf->pubsetbuf(buffer_data, buffer_length);

      // Read the float format
      oct_mach_info::float_format flt_fmt;

      recv_buffer.read(X_CAST(char *,&flt_fmt),
                       sizeof(oct_mach_info::float_format));

      // Convert binary buffer_data back to an octave value
      bool global;
      octave_value recv_value;
      std::string help;
      std::string var_name;

      while(!recv_buffer.eof())
        {
          var_name = mpi_read_binary_data (recv_buffer, false,
                                           flt_fmt,
                                           "", global, recv_value, help);
          if(var_name.empty())
            break;
	  else
            retr_list.append(recv_value);
	  
        }

      return true;
    }
  catch(std::string e)
    {
      kill(poctave_common::mpi_pid,SIGURG);
      return false;
    }
};

DEFUN_DLD(mpi_bcast, args, ,
	  "-*- texinfo -*-\n\
@deftypefn {Loadable Function} {[@var{status}]=} mpi_bcast ( @var{msg}, @var{size}, @var{rank}, @var{comm} )\n\
@deftypefnx {Loadable Function} {[@var{status}, @var{msg}]=} mpi_bcast (@var{size}, @var{comm}, @var{rank} )\n\
Broadcasts a message from the process with rank \"root\" to all other processes of the group.\n\
INPUT PARAMETERS\n\
\n\
@table @code\n\
@item @var{msg}\n octave_value to broadcast (scalar)\n\
@item @var{size}\n size of octave_value to bloadcast (scalar)\n\
@item @var{rank}\n rank of broadcast root (scalar)\n\
@item @var{comm}\n communicator (scalar)\n\
@end table\n\
\n\
OUTPUT PARAMETERS\n\
\n\
@table @code\n\
@item @var{status}\n received messages (octave_value)\n\
@item @var{msg}\n received messages (octave_value)\n\
@end table\n\
\n\
@example\n\
@group\n\
\n\
@end group\n\
@end example\n\
@end deftypefn")
{
  octave_value_list retval;

  if( ( args.length() != 3 ) && ( args.length() != 4 ) )
    {
      print_usage("mpi_bcast");
      return retval;
    }

  if( ! poctave_common::poctave_is_active )
    {
      error("mpi_init must be called beforehand");
      return retval;
    }

  if( args.length() == 4 )
    {
      // Sending operation
      mpi_bcast_root(args,retval);
    }
  else
    {
      // Reciving operation
      mpi_bcast_non_root(args,retval);
    }

  return retval;  
}

