#include <octave/oct.h>
#include <defun-dld.h>
#include <error.h>
#include <ov.h>

#include "libdld_common.h"
#include "fifo_host.h"

DEFUN_DLD (mpi_cancel, args, ,
           "-*- texinfo -*-\n\
@deftypefn {Loadable Function} { [ @var{stat} ] =} mpi_cancel ( @var{req} )\n\
Cancels a communication request\n\
\n\
INPUT PARAMETERS\n\
\n\
@table @code\n\
@item @var{req}\n communication request (scalar)\n\
@end table\n\
\n\
OUTPUT PARAMETERS\n\
\n\
@table @code\n\
@item @var{stat}\n If an error occures, false is returned. otherwise true is returned (scalar) \n\
@end table\n\
\n\
@example\n\
@group\n\
\n\
For example,\n\
\n\
tag = 0; dest = 0; src = 0;\n\
req = mpi_irecv ( sizeof(\"hello world\"), src, tag, MPI_COMM_WORLD )\n\
stat = mpi_cancel( req )\n\
\n\
@end group\n\
@end example\n\
@end deftypefn")
{
  octave_value_list retval;

  if( args.length() != 1 ){
    print_usage("mpi_cancel");
    error("invalid argument");
    retval(0) = false;
    return retval;
  }

  if( ! poctave_common::poctave_is_active )
    {
      error("mpi_init must be called beforehand.");
      retval(0) = false;
      return retval;
    }

  int key; //Request

  if( args(0).is_real_scalar() )
    {
      key=args(0).int_value();
    }
  else
    {
      error("request must be scalar");
      retval(0) = false;
      return retval;
    }

  fifo_host *mpiproc=(fifo_host *)poctave_common::mpiproc;

  try{
    mpiproc->send("mpi_cancel",sizeof("mpi_cancel"));
    
    mpiproc->send(&key,sizeof(int));
    
    int mpi_retval;
    mpiproc->recv(&mpi_retval,sizeof(int));
    
    if( mpi_retval == 0)
      retval(0) = true;
    else
      retval(0) = false;
  }
  catch(std::string e)
    {
      kill(poctave_common::mpi_pid,SIGURG);
      error("operation canceled");
      retval(0) = false;
      return retval;
    }

  return retval;
}

