#include <octave/oct.h>
#include <defun-dld.h>
#include <error.h>
#include <ov.h>

#include "libdld_common.h"
#include "fifo_host.h"

DEFUN_DLD (mpi_comm_rank, args, ,
          "-*- texinfo -*-\n\
@deftypefn {Loadable Function} { [@var{rank}] =} mpi_comm_rank ( @var{comm} )\n\
Determines the rank of the calling process in the communicator\n\
\n\
INPUT PARAMETERS\n\
\n\
@table @code\n\
@item @var{comm}\n communicator (scalar)\n\
@end table\n\
\n\
OUTPUT PARAMETERS\n\
\n\
@table @code\n\
@item @var{rank}\n rank of the calling process in group of comm (scalar)\n\
@end table\n\
\n\
@example\n\
@group\n\
\n\
For example,\n\
\n\
my_rank = mpi_comm_rank ( MPI_COMM_WORLD )\n\
\n\
@end group\n\
@end example\n\
@end deftypefn")
{
  
  if(args.length()!=1){
    print_usage("mpi_comm_rank");
    return octave_value();
  }

  if(!poctave_common::poctave_is_active){
    return octave_value("mpi_init must be called beforehand");
  }

  int key; //COMMUNICATOR;

  if(args(0).is_real_scalar()){
    key=args(0).int_value();
  }
  else{
    error("mpi_comm_rank: bad reciever");
    return octave_value();
  }

  fifo_host *mpiproc=(fifo_host *)poctave_common::mpiproc;

  mpiproc->send("mpi_comm_rank",sizeof("mpi_comm_rank"));
  
  mpiproc->send(&key,sizeof(int));
  int rank;
  mpiproc->recv(&rank,sizeof(int));

  return octave_value(rank);
};
