#include <octave/oct.h>
#include <defun-dld.h>
#include <error.h>
#include <ov.h>

#include "libdld_common.h"
#include "fifo_host.h"

DEFUN_DLD (mpi_comm_size, args, ,
          "-*- texinfo -*-\n\
@deftypefn {Loadable Function} { [@var{size}] =} mpi_comm_size ( @var{comm} )\n\
Determines the size of the group associated with a communictor.\n\
\n\
INPUT PARAMETERS\n\
\n\
@table @code\n\
@item @var{comm}\n communicator (scalar)\n\
@end table\n\
\n\
OUTPUT PARAMETERS\n\
\n\
@table @code\n\
@item @var{size}\n the  size of the group associated with a communictor (scalar)\n\
@end table\n\
\n\
@example\n\
@group\n\
\n\
For example,\n\
\n\
size = mpi_comm_size ( MPI_COMM_WORLD )\n\
\n\
@end group\n\
@end example\n\
@end deftypefn")
{

  if(args.length()!=1){
    print_usage("mpi_comm_size");
    return octave_value();
  }

  if(!poctave_common::poctave_is_active){
    return octave_value("mpi_init must be called beforehand.");
  }

  int key; //COMMUNICATOR;

  if(args(0).is_real_scalar()){
    key=args(0).int_value();
  }
  else{
    error("mpi_comm_size: bad reciever");
    return octave_value();
  }

  fifo_host *mpiproc=(fifo_host *)poctave_common::mpiproc;
  
  mpiproc->send("mpi_comm_size",sizeof("mpi_comm_size"));
  
  mpiproc->send(&key,sizeof(int));

  int size;
  mpiproc->recv(&size,sizeof(int));

  return octave_value(size);
};

