#include <octave/oct.h>
#include <octave/oct.h>
#include <defun-dld.h>
#include <error.h>
#include <ov.h>
#include <signal.h>

#include "libdld_common.h"
#include "fifo_host.h"
#include "process_info.h"

DEFUN_DLD (mpi_finalize, args, ,
	   "-*- texinfo -*-\n\
@deftypefn {Loadable Function} mpi_finalize \n\
\n\
Terminates MPI execution environment.\n\
@example\n\
@group\n\
\n\
For example,\n\
\n\
number_of_processes = 3;\n\
mpi_init( number_of_processes );\n\
...\n\
mpi_finalize();\n\
@end group\n\
@end example\n\
@end deftypefn")
{
  if(!poctave_common::poctave_is_active){
    print_usage("mpi_finalize");
    return octave_value("mpi_init must be called beforehand");
  }

  //Stop remote wrap & octave process
  char command[512];
  int sendsize;
  strncpy(command,"poctave_exit",sizeof("poctave_exit"));
  sendsize=strlen(command);

  process_info<node_process_info *>::iterator i;
  for(i=process_info<node_process_info *>::instance()->begin();
      i!=process_info<node_process_info *>::instance()->end();
      i++){
    if((*i)->mpi_rank!=0){
      std::string commret(command);
      poctave_common::host->sendhost((*i)->hostname.c_str(),
				     (*i)->wrapper_local_to_remote_pid,
				     command,sendsize);
      wait(NULL);
    }
  }
  
  //Close session to wrapper
  poctave_common::host->stop();
  
  //Stop local mpi process

  fifo_host *mpiproc=(fifo_host *)poctave_common::mpiproc;

  kill(poctave_common::mpi_pid,SIGURG);
  mpiproc->send("EXIT",sizeof("EXIT"));
  wait(NULL);
  
  poctave_common::poctave_is_active=false;
  
  //delete objects
  delete poctave_common::host;
  poctave_common::host=NULL;
  delete poctave_common::mpiproc;
  poctave_common::mpiproc=NULL;

  process_info<node_process_info *>::iterator it;
  for(it=process_info<node_process_info *>::instance()->begin();
      it!=process_info<node_process_info *>::instance()->end();it++){ 
    delete (*it);
  }

  process_info<node_process_info *>::instance()->clear();

  return octave_value("poctave closed");
};



