#include <octave/oct.h>
#include <octave/byte-swap.h>

#include "libdld_common.h"
#include "fifo_host.h"

DEFUN_DLD(mpi_irecv, args, ,
	  "-*- texinfo -*-\n\
@deftypefn {Loadable Function} {[@var{request}]=} mpi_irecv ( @var{size}, @var{src}, @var{tag}, @var{comm} )\n\
Begins a non-blocking receive.\n\
\n\
INPUT PARAMETERS\n\
\n\
@table @code\n\
@item @var{size}\n size of octave_value_list to receive (scalar)\n\
@item @var{src}\n rank of source (scalar)\n\
@item @var{tag}\n message tag (scalar)\n\
@item @var{comm}\n communicator (scalar)\n\
@end table\n\
\n\
OUTPUT PARAMETERS\n\
\n\
@table @code\n\
@item @var{request}\n communication request (scalar)\n\
@end table\n\
\n\
@example\n\
@group\n\
\n\
For example,\n\
\n\
tag = 0; dest = 0; src = 0;\n\
req = mpi_irecv ( sizeof(\"hello world\"), src, tag, MPI_COMM_WORLD )\n\
if ( mpi_test( req ) )\n\
\t mpi_wait( req )\n\
\n\
@end group\n\
@end example\n\
@end deftypefn")
{
  octave_value_list retval;

  if( args.length() != 4 )
    {
      print_usage("mpi_irecv");
      return retval;
    }

  if( ! poctave_common::poctave_is_active )
    return octave_value("mpi_init must be called beforehand");

  int size;
  int src;
  int tag;
  int key; //COMMUNICATOR

  if( args(0).is_real_scalar() &&
      args(1).is_real_scalar() &&
      args(2).is_real_scalar() &&
      args(3).is_real_scalar() )
    {
      size=args(0).int_value();
      src =args(1).int_value();
      tag =args(2).int_value();
      key =args(3).int_value();
    }
  else
    {
      error("mpi_recv: bad operants");
      return retval;
    }

  fifo_host *mpiproc=(fifo_host *)poctave_common::mpiproc;

  try
    {
      mpiproc->send("mpi_irecv",sizeof("mpi_irecv"));
      
      mpiproc->send(&size,sizeof(int));
      mpiproc->send(&src,sizeof(int));
      mpiproc->send(&tag,sizeof(int));
      mpiproc->send(&key,sizeof(int));
      
      int key_req;
      mpiproc->recv(&key_req,sizeof(int));

      if ( key_req == -1 )
	error("mpi_recv: Error in MPI_Irecv");
      else
	retval.append(octave_value(key_req));
    }
  catch(std::string e)
    {
      kill(poctave_common::mpi_pid,SIGURG);
      return octave_value(e);
    }
  
  return retval;  
}

