#include <octave/oct.h>
#include <defun-dld.h>
#include <error.h>
#include <ov.h>

#include "libdld_common.h"
#include "fifo_host.h"

DEFUN_DLD (mpi_test, args, ,
	   "-*- texinfo -*-\n\
@deftypefn {Loadable Function} { [ @var{flag} @var{msg} ] =} mpi_test ( @var{req} )\n\
Tests for the completion of a send or receive\n\
\n\
INPUT PARAMETERS\n\
\n\
@table @code\n\
@item @var{req}\n communication request (scalar)\n\
@end table\n\
\n\
OUTPUT PARAMETERS\n\
\n\
@table @code\n\
@item @var{flag}\n true if operation completed. If operation is not completed, or an error occures, false is returned. (bool)\n\
@item @var{msg}\n received message is set if @var{flag} is true. If @var{flag} is false, error message is set.  (octave_value)\n\
@end table\n\
\n\
@example\n\
@group\n\
\n\
For example,\n\
\n\
tag = 0; dest = 0; src = 0;\n\
mpi_send ( sizeof(\"hello world\"), dest, tag, MPI_COMM_WORLD )\n\
req = mpi_irecv ( sizeof(\"hello world\"), src, tag, MPI_COMM_WORLD )\n\
[flag msg] = mpi_test( req );\n\
if ( flag )\n\
  printf(\"%s\",msg);\n\
else\n\
  error(msg);\n\
endif\n\
\n\
@end group\n\
@end example\n\
@end deftypefn")
{
  octave_value_list retval;

  if( args.length() != 1 )
    {
      print_usage("mpi_test");
      retval(0) = octave_value(false);
      retval(1) = std::string("invalid argument");
      return retval;
    }

  if( ! poctave_common::poctave_is_active )
    {
      retval(0) = octave_value(false);
      retval(1) = std::string("mpi_init must be called beforehand.");
      return retval;
    }

  int key; //Request

  if(args(0).is_real_scalar()){
    key=args(0).int_value();
  }
  else
    {
      error("request must be scalar");
      retval(0) = false;
      retval(1) = std::string("request must be scalar");
      return retval;
    }
  
  fifo_host *mpiproc=(fifo_host *)poctave_common::mpiproc;

  try
    {
      mpiproc->send("mpi_test",sizeof("mpi_test"));
      
      mpiproc->send(&key,sizeof(int));
      
      int mpi_retval;
      mpiproc->recv(&mpi_retval,sizeof(int));
      
      retval(0) = mpi_retval;
      retval(1) = true;
    }
  catch(std::string e)
    {
      kill(poctave_common::mpi_pid,SIGURG);
      retval(0) = false;
      retval(1) = std::string("operation canceled");
      return retval;
    }
  
  return retval;
}
