#include <octave/oct.h>
#include <defun-dld.h>
#include <error.h>
#include <ov.h>
#include <variables.h>
#include <parse.h>
#include <toplev.h>

#include "fifo_host.h"
#include "log.h"
#include "fileplace.h"
#include "libdld_common.h"

DEFUN_DLD (remote_command_server, args, ,
                   "test (hosts)\n\
                   \n\
                   Connect hosts and return sockets.")
{
  if( poctave_common::poctave_is_active ){
    return octave_value("octave is already invoked");
  }

  //Search mpi_rank which this(octave) process connect to
  symbol_record *sr = top_level_sym_tab->lookup ("fileplace_prefix");

  if (!sr)
    assert(false);
  else
    {      
      octave_value def;
      def = sr->def();

      std::string fileplace_prefix = def.string_value();

      if(fileplace_prefix.empty())
	assert(false);
      else
	Fileplace::set_prefix(fileplace_prefix);
    }

  poctave_common::poctave_is_active=true;

  char hostname[512];
  gethostname(hostname,sizeof(hostname));

  //Create log file
  std::string logpath=Fileplace::getpath()+hostname+string("_octave.log");
  Log *log=new Log(logpath.c_str(),O_CREAT|O_APPEND|O_WRONLY);
  assert(log!=NULL);
  
  //Connect to local wrap process
  std::string wrapfifopath1=Fileplace::getpath()+hostname+"wrapfifo1";
  std::string wrapfifopath2=Fileplace::getpath()+hostname+"wrapfifo2";

  fifo_host *wrapproc =
    new fifo_host(wrapfifopath2,wrapfifopath1,1);

  assert(wrapproc!=NULL);
  wrapproc->start();
  log->writelog("Connected to local wrapper process");

  //Connect to local mpi process
  std::string mpififopath1=Fileplace::getpath()+hostname+"magefifo";
  std::string mpififopath2=Fileplace::getpath()+hostname+"hogefifo";
  poctave_common::mpiproc=new fifo_host(mpififopath1,mpififopath2,1);
  assert(poctave_common::mpiproc!=NULL);
  poctave_common::mpiproc->start();

  log->writelog("Connected to local mpir process");

  //Recv pid from mpi process
  poctave_common::mpiproc->recv(&poctave_common::mpi_pid,
				sizeof(poctave_common::mpi_pid));
  
  //Send my process-id to wrap (for synchonize in mpi_init.oct)
  pid_t value=getpid();
  wrapproc->send(&value,sizeof(pid_t));
  
  log->writelog("synchonize finish");

  //Processing loop
  while(true)
    {
      char buff[512];
      memset(buff,'\0',sizeof(buff));
      int size=wrapproc->recv(buff,sizeof(buff));
      buff[size+1]=';';
      std::string ope(buff);

      if( strncmp(buff,"poctave_exit",sizeof("poctave_exit")) == 0 ){
	//If recv "poctave_exit" command (send by mpi_finalize.oct)
	//Kill connected mpi process and end loop

	kill(poctave_common::mpi_pid,SIGURG);
	poctave_common::mpiproc->send("EXIT",sizeof("EXIT"));
	break;
      }

      log->writelog("evaluate %s",buff);
      int stat;
      eval_string(ope,true,stat);
      log->writelog("%s fin,error_state=%d",buff,error_state);
      error_state=false;
    }
  
  //Delete objects
  delete log;
  log=NULL;
  delete wrapproc;
  wrapproc=NULL;
  delete poctave_common::mpiproc;
  poctave_common::mpiproc=NULL;
  
  clean_up_and_exit (EXIT_SUCCESS);
};


