#include <octave/oct.h>
#include <defun-dld.h>
#include <parse.h>

#include "libdld_common.h"
#include "process_info.h"

DEFUN_DLD (reval, args, ,
          "-*- texinfo -*-\n\
@deftypefn {Loadable Function} reval ( @var{command} , @var{rank} )\n\
Evaluate @var{command} at the remote process specified by @var{rank}.\n\
\n\
INPUT PARAMETERS\n\
\n\
@table @code\n\
@item @var{command}\n command to evaluate (string)\n\
@item @var{rank}\n process rank (scalar)\n\
@end table\n\
\n\
@example\n\
@group\n\
\n\
For example,\n\
\n\
mpi_init( 2 );\n\
\n\
reval ( \"fprintf( stderr, \\\"hello world\\\" )\", 1);\n\
\n\
mpi_finalize\n\
\n\
@end group\n\
@end example\n\
@end deftypefn")
{
  octave_value_list retval;
  
  if( args.length() != 2)
    {
      print_usage("reval");
      return retval;
    }
  
  int send_mpi_rank;
  std::string command_arg;
  
  if( args(0).is_char_matrix() && args(1).is_real_scalar())
    {      
      command_arg = args(0).string_value();
      send_mpi_rank = args(1).int_value();
    }
  else
    {
      error("invalid format");
      return retval;
    }

  if(send_mpi_rank != 0){
    bool eval_fin=false;
    process_info<node_process_info *>::iterator i;
    for(i = process_info<node_process_info *>::instance()->begin();
	i != process_info<node_process_info *>::instance()->end();
	i++ )
      {
	if( (*i)->mpi_rank == send_mpi_rank ){
	  poctave_common::host->sendhost((*i)->hostname,
					 (*i)->wrapper_local_to_remote_pid,
					 (char *)command_arg.c_str(),
					 command_arg.size());
	  eval_fin=true;
	}
      }    
    if( ! eval_fin )
      {
	error("There is no rank \"%d\"", send_mpi_rank);
	return retval;
      }
  }
  else{
    /*
     * exective toward user_controling process;
     */
    int ret;
    eval_string(command_arg, false, ret, 1);
    return retval;
  }

  return retval;

};
