#include <octave/oct.h>
#include <octave/byte-swap.h>

#include "libdld_common.h"

DEFUN_DLD(sizeof,args , ,
	  "-*- texinfo -*-\n\
@deftypefn {Loadable Function} { [ @var{size} ] =} sizeof ( @var{msg} )\n\
@deftypefnx {Loadable Function} { [ @var{size} ] =} sizeof \n\
\n\
Measures size of binary comverted octave_value.\n\
\n\
@var{msg} octave_value to measure (octave_value)\n\
@example\n\
@group\n\
\n\
For example,\n\
\n\
mpi_init( 2 );\n\
tag = 0; dest = 1; src = 0;\n\
rank = mpi_comm_rank;\n\
if ( rank == 0 )\n\
  msg = rand( 5 );\n\
  mpi_send( sizeof( msg ), sizeof, dest, tag,MPI_COMM_WORLD);\n\
  mpi_send( msg , sizeof( msg ), dest, tag,MPI_COMM_WORLD);\n\
else\n\
  size = mpi_recv( sizeof, src, tag,MPI_COMM_WORLD);\n\
  msg = mpi_recv( size , src , tag,MPI_COMM_WORLD);\n\
endif\n\
mpi_finalize;\n\
@end group\n\
@end example\n\
@end deftypefn")
{

  if( args.length() == 0 )
    {
      std::basic_ostringstream<char> buffer;
      
      // Save the byte format to the stream
      oct_mach_info::float_format flt_fmt =
	oct_mach_info::native_float_format();
      
      buffer.write(X_CAST(char *,&flt_fmt),
		   sizeof(oct_mach_info::float_format));
      
      // Serialize all each entry in binary form
      // Convert octave_value to binary
      mpi_save_binary_data(buffer,
			   octave_value(sizeof(int)), "V", "",
			   false, false);
      buffer.eof();
      
      std::basic_string<char> data = buffer.str();
      int buffer_length = data.length();
      return octave_value(buffer_length);
    }
  
  std::basic_ostringstream<char> buffer;
  
  // Save the byte format to the stream
  oct_mach_info::float_format flt_fmt =
    oct_mach_info::native_float_format();

  buffer.write(X_CAST(char *,&flt_fmt),
	       sizeof(oct_mach_info::float_format));
  
  // Serialize all each entry in binary form
  for (int i = 0; i< args.length() ; i++)
    {
      // Make sure the value exists
      if (args(i).is_defined())
        {
          // Convert octave_value to binary
          mpi_save_binary_data(buffer,
                               args(i), "V", "",
                               false, false);
        }
      else
        {
          error("mpi_send: Doesn't handle that form of data yet");

          return octave_value();
        }
    }

  buffer.eof();
  
  std::basic_string<char> data = buffer.str();
  int buffer_length = data.length();

  return octave_value(buffer_length);

}
