#include "log.h"

#include <cstdarg>
#include <unistd.h>
#include <fcntl.h>
#include <sys/timeb.h>
#include <errno.h>
#include <stdio.h>
#include <assert.h>

Log::Log(const string & o_filename,const int flags){
  string filename=o_filename;
  if(creat(filename.c_str(),flags)<0){
    perror("ERROR Log::Log creat");
    assert(false);
  }

  chmod(filename.c_str(),S_IRUSR|S_IWUSR);
  fd=open(filename.c_str(),flags);
  if(fd==-1)
    fd=1;
  if(fd==-1){
    perror("ERROR Log::Log");
    fprintf(stderr,"[%s] in process %d\n",filename.c_str(),getpid());
    assert(false);
  };

  char c_hostname[512];
  if(gethostname(c_hostname,sizeof(c_hostname))<0){
    perror("ERROR Log::Log");
    exit(EXIT_FAILURE);
  }
  hostname=c_hostname;

  writelog("INFO:===Start logging===");
};
     
void Log::nfslock(void){
  //NOT IMPLEMENTED
};
void Log::nfsunlock(void){
  //NOT IMPLEMENTED
};

bool Log::writelog(char *fmt,...){
  struct flock lock;  
  //Lock 
  lock.l_type = F_WRLCK;
  lock.l_whence = SEEK_SET;
  lock.l_start = (off_t) 0;
  lock.l_len = (off_t) 0;
  if(fcntl(fd,F_SETLKW,&lock)<0){
    if(errno==ENOLCK){
      errno=0;
      nfslock();
    }
    else{
      perror("ERR Log::writelog");
      return false;
    }
  }

  //Print log 
  // [(host)] pid=(pid),ppid=(ppid),sid=(sid) (date)
  // (message)
  va_list ap;
  va_start(ap,fmt);

  char buff[512];
  size_t len;
  pid_t pid;
  pid_t ppid;
  pid_t sid;
  pid=getpid();
  ppid=getppid();
  sid=getsid(0);
  time_t currtime;
  time(&currtime);
  char *pt=ctime(&currtime);
  
  len=snprintf(buff,sizeof(buff),"[%s] pid=%5d,ppid=%5d,sid=%5d %s",
	       hostname.c_str(),pid,ppid,sid,pt);
  write(fd,buff,len-1);

  char space[]="  :";
  write(fd,space,strlen(space));

  fsync(fd);

  len=vsnprintf(buff,sizeof(buff),fmt,ap);
  va_end(ap);

  for(unsigned int i=0;i<sizeof(buff);i++){
    if(buff[i]=='\n'){
      char ret=0xa;
      buff[i]=ret;
    }
  }

  write(fd,buff,len);
  char ret='\n';
  write(fd,&ret,sizeof(char));
  fsync(fd);

  //Unlock
  lock.l_type=F_UNLCK;
  lock.l_whence=SEEK_SET;
  lock.l_start=(off_t) 0;
  lock.l_len=(off_t) 0;
  if (fcntl(fd,F_SETLKW, &lock)<0){
    if(errno==ENOLCK){
      nfsunlock();
      errno=0;
    }
    else{
      perror("ERR Log::writelog");
      return false;
    }
  };

  return true;
};
  
Log::~Log(void){
  writelog("INFO:===Stop logging===");
  close(fd);
};

