#include"netbase.h"
#include"log.h"

char* Netbase::getnamebyfd(int fd){
  sockaddr_in r_addr;
  struct hostent *hehe;
  if(fd<=0){
    return NULL;
  }
  socklen_t len = sizeof(r_addr);  
  getpeername(fd, (sockaddr*)&r_addr, &len );
  hehe=gethostbyaddr((char *)&r_addr.sin_addr.s_addr,sizeof(r_addr.sin_addr), AF_INET);  
  return hehe->h_name;
};

int Netbase::scan_any_stream(std::string &remote_hostname,
			     pid_t &remote_pid,
			     int waittime)
{
  struct pollfd *stream = (struct pollfd *)
    malloc(sizeof(struct pollfd) * hostlist.size() );

  for(unsigned int i = 0; i<hostlist.size(); i++)
    {
      stream[i].fd=hostlist[i]->fdr;
      stream[i].events=POLLIN|POLLERR|POLLHUP|POLLNVAL;
    }
  
  int ret = poll(stream,hostlist.size(),waittime);
  
  if( ret == 0 )
    {
      //no input stream
      return 0;
    }
  else if( ret < 0)
    {
      if(errno == EINTR)
	//throw std::string("Netbase::scanstream interrupted by signal");    
	
      return -1;
    }

  //detect stream or error in socket
  for(unsigned int i = 0; i < hostlist.size(); i++){
    
    remote_hostname = hostlist[i]->hostname;
    remote_pid = hostlist[i]->pid;

    if((stream[i].revents==POLLERR)||
       (stream[i].revents==POLLHUP)||
       (stream[i].revents==POLLNVAL))
      {
	//throw std::string("Netbase::scanstream Bad file discripter");
	return -1;
      }
    if( stream[i].revents == POLLIN )
      {
	return 1;
      }
  }
  
  return 1;
}  
  
int Netbase::atoaddr(const char *address,struct in_addr *addr)
{
  struct hostent *host;
  addr->s_addr = inet_addr(address);

  if( (signed int)addr->s_addr != -1 )
    {
      return 1;
    }

  host = gethostbyname(address);

  if( host != NULL )
    {
      struct in_addr *aptr;
      aptr = (struct in_addr *)*host->h_addr_list;
      memcpy(addr,aptr,sizeof(struct  in_addr));
      return 1;
    }
  
  return 0;
};

int Netbase::atoport(const char *port,char *tcp_udp)
{
  struct servent *se;
  int portno;

  if( (se = getservbyname(port,tcp_udp)) == NULL)
    {
      if((portno=atoi(port))==0){
	return 0;
      }
      errno=0;
      return htons(portno);
    }
  else
    {
      return se->s_port;
    }
  
  return 0;
};

int Netbase::socketclose(int soc)
{
  int ret;
  shutdown(soc,2);
  ret = close(soc);
  return ret;
};

int Netbase::recvsize(int soc,char *buf,int size){

  ssize_t len,lest;
  char *ptr;
  lest=size;
  ptr=buf;

  while(true){

    len=read(soc,ptr,lest);

    if(len == -1){

      char hostname[512];
      pid_t pid;
      pid=getpid();
      gethostname(hostname,sizeof(hostname));
      char buff[512];
      sprintf(buff,"ERR Netbase::recvsize:(pid=%d,hostname=%s) error in recv",
              pid,
              hostname);
      //throw std::string(buff);
      return -1;
    }

    if(len == 0){
      char hostname[512];
      pid_t pid;
      pid=getpid();
      gethostname(hostname,sizeof(hostname));
      char buff[512];
      sprintf(buff,"ERR Netbase::recvsize:(pid=%d,hostname=%s) file end",
              pid,
              hostname);
      return -1;
      //throw std::string(buff);
    }

    lest-=len;
    ptr+=len;

    if(lest<=0){
      break;
    }

  }

  return size;

};

int Netbase::sendsize(int soc,char *buf,int size){
  int lest,len;
  char *ptr;
  lest=size;
  ptr=buf;

  while(1){
    len=write(soc,ptr,lest);
    if(len<0){
      char hostname[512];
      pid_t pid;
      pid=getpid();
      gethostname(hostname,sizeof(hostname));
      char buff[512];
      sprintf(buff,"ERR Netbase::sendsize:(pid=%d,hostname=%s)",
              pid,
              hostname);
      //throw std::string(buff);
      return -1;
    }
    lest-=len;
    ptr+=len;
    if(lest<=0){
      break;
    }
  }
  return size;
};

bool Netbase::connectedhost(host *h){

  hostlist.push_back(h);

  return true;
};

Netbase::Netbase(int p) : portnumber(p){
};

bool Netbase::sendhost(std::string hostname,
		       pid_t pid,const void *val,
		       int valsize){
  std::vector<host *>::iterator itr;
  for(itr=hostlist.begin();itr!=hostlist.end();itr++){
    if( ((*itr)->hostname == hostname) && ((*itr)->pid == pid ))
      {
	sendsize((*itr)->fdw,(char *)&valsize,sizeof(int));
	sendsize((*itr)->fdw,(char *)val,valsize);
	return true;
      }
  }

  return false;
};

int Netbase::recvhost(std::string hostname,pid_t pid,const void *val,int valsize){
  std::vector<host *>::iterator itr;
  for(itr=hostlist.begin();itr!=hostlist.end();itr++){
    if( ((*itr)->hostname == hostname ) && ((*itr)->pid==pid)){
      int recvlen;
      recvsize((*itr)->fdr,(char *)&recvlen,sizeof(int));
      if(recvlen>valsize){
	fprintf(stderr,"ERR -Recv buffer is too short-");
	return -1;
      }
      memset((char *)val,'\0',valsize);
      recvsize((*itr)->fdr,(char *)val,recvlen);

      return recvlen;
    }
  }
  return -1;
};

bool Netbase::stop(void){

  vector<host *>::iterator itr;

  for(itr=hostlist.begin();itr!=hostlist.end();itr++){
    if((*itr)->fdw==(*itr)->fdr)
      close((*itr)->fdw);
    else{
      close((*itr)->fdr);
      close((*itr)->fdw);
    }
    delete *itr;
  }

  hostlist.clear();
  return true;
};

bool Netbase::send(const void *val,int valsize){
  
  sendsize(hostlist[0]->fdw,(char *)&valsize,sizeof(int));
  sendsize(hostlist[0]->fdw,(char *)val,valsize);
  return true;
};


int Netbase::recv(const void *val,int valsize){
  int recvlen;

  recvsize(hostlist[0]->fdr,(char *)&recvlen,sizeof(int));

  if(recvlen>valsize){
    //throw std::string("ERR -Recv buffer is too short-");
    return -1;
  }
  memset((char *)val,'\0',valsize);

  recvsize(hostlist[0]->fdr,(char *)val,recvlen);
  
  return recvlen;
};

Netbase::~Netbase(void){
  stop();
};
