#include <unistd.h>
#include <stdio.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <map>
#include <list>
#include <setjmp.h>
#include <sstream>
#include <signal.h>

#include "mpi_module.h"
#include "log.h"
#include "fifo_host.h"
#include "fileplace.h"
#include "process_info.h"
#include "global.h"

using namespace std;
fifo_host *octaveproc=NULL;

int regist(void){
  Function::setfd(octaveproc);
  Handler::registfunc(new Function_mpi_comm_rank());
  Handler::registfunc(new Function_mpi_send());
  Handler::registfunc(new Function_mpi_recv());
  Handler::registfunc(new Function_mpi_comm_size());
  Handler::registfunc(new Function_mpi_irecv());
  Handler::registfunc(new Function_mpi_cancel());
  Handler::registfunc(new Function_mpi_test());
  Handler::registfunc(new Function_mpi_wait());
  Handler::registfunc(new Function_mpi_testany());
  Handler::registfunc(new Function_mpi_waitany());
  Handler::registfunc(new Function_mpi_testall());
  Handler::registfunc(new Function_mpi_waitall());
  Handler::registfunc(new Function_mpi_bcast());
  Handler::registfunc(new Function_mpi_barrier());
  return 1;
};

sigjmp_buf loopindex;

void cancel_operation(int sig){
  signal(SIGURG,cancel_operation);
  siglongjmp(loopindex,1);
};

bool loop(Log *log,Netbase *pipe){

  //string remotehostname=pipe->getremotehostname(0);
  
  if(sigsetjmp(loopindex,1)!=0){
    //operation canceled
    log->writelog("INFO:Operaton cancel");
  }
  signal(SIGURG,cancel_operation);      
  while(true){
    char operation[1024];
    log->writelog("INFO:Scanning");

    //NEEDFIX replace function
    std::string remotehostname;
    pid_t remotepid;
    //pipe->scanstream(remotehostname,-1);
    pipe->scan_any_stream(remotehostname,remotepid,-1);

    log->writelog("INFO:Detect stream from [%s]",remotehostname.c_str());
    pipe->recv(operation,sizeof(operation));
    log->writelog("INFO:recv stream [%s]",operation);

    if((string)operation==(string)"EXIT"){
      log->writelog("INFO:exit loop");
      return true;
    }
    
    string retval=Handler::handle((string)operation);
    log->writelog("INFO:Process [%s] complete,%s",operation,retval.c_str());
  }
  
  return true;
};

struct hostinfo{
  string hostname;
  pid_t mpi_pid;
  int mpi_rank;
};

list<hostinfo *> hostinfotable;

bool sendhostinfo(void){
  int rank;
  int size;
  int mpi_pid;
  char hostname[512];
  
  memset(hostname,'\0',sizeof(hostname));
  gethostname(hostname,sizeof(hostname));
  mpi_pid=getpid();

  MPI_Comm_rank(MPI_COMM_WORLD,&rank);
  MPI_Comm_size(MPI_COMM_WORLD,&size);
  int tag=1;
  if(rank==0){
    /*
     * NEEDFIX 
     * We assume that process which MPI_Comm_rank 0 starts other octave.
     * Shareing host information between all MPI_COMM_WORLD is more desirable.
     */

    mpi_process_info *tmp=new mpi_process_info;

    tmp->hostname=string(hostname);
    tmp->mpi_pid=mpi_pid;
    tmp->mpi_rank=rank;

    process_info<mpi_process_info *>::instance()->push_back(tmp);

    for(int i=1;i<size;i++){
      MPI_Status stat;
      MPI_Recv(&rank,1,MPI_INT,i,tag,MPI_COMM_WORLD,&stat);
      MPI_Recv(&mpi_pid,1,MPI_INT,i,tag,MPI_COMM_WORLD,&stat);
      MPI_Recv(hostname,sizeof(hostname),MPI_CHAR,i,tag,MPI_COMM_WORLD,&stat);

      tmp=new mpi_process_info;
      tmp->hostname=string(hostname);
      tmp->mpi_pid=mpi_pid;
      tmp->mpi_rank=rank;

      process_info<mpi_process_info *>::instance()->push_back(tmp);
    }
  }
  else{
    MPI_Send(&rank,1,MPI_INT,0,tag,MPI_COMM_WORLD);
    MPI_Send(&mpi_pid,1,MPI_INT,0,tag,MPI_COMM_WORLD);
    MPI_Send(hostname,sizeof(hostname),MPI_CHAR,0,tag,MPI_COMM_WORLD);
  }
  return true;
};

//  void test_error(void){
//    char hostname[512];
//    int tag=0;
//    MPI_Send(hostname,sizeof(hostname),MPI_CHAR,-1,tag,MPI_COMM_WORLD);
//  };

int main(int argc,char *argv[]){

  MPI_Init(&argc,&argv);
  int rank;
  MPI_Comm_rank(MPI_COMM_WORLD,&rank);

  std::stringstream sstrm;
  sstrm<<rank;
  Fileplace::set_prefix(sstrm.str());

  char hostname[512];
  gethostname(hostname,sizeof(hostname));
  string filename=Fileplace::getpath()+hostname+"_mpi.log";
  Log *log=new Log(filename,O_CREAT|O_WRONLY|O_APPEND);

  sendhostinfo();

  string fifopath1=Fileplace::getpath()+hostname+"magefifo";
  string fifopath2=Fileplace::getpath()+hostname+"hogefifo";  

  octaveproc=new fifo_host(fifopath2,fifopath1,2);
  octaveproc->start();
  pid_t mypid=getpid();
  octaveproc->send(&mypid,sizeof(pid_t));

  if(rank==0){
    //NEEDFIX really always rank==0 ?

    int hostsize=process_info<mpi_process_info *>::instance()->size();
    octaveproc->send(&hostsize,sizeof(int));

    process_info<mpi_process_info *>::iterator i;

    for(i=process_info<mpi_process_info *>::instance()->begin();
	i!=process_info<mpi_process_info *>::instance()->end();i++){
      octaveproc->send(((*i)->hostname.c_str()),(*i)->hostname.size());
      octaveproc->send(&((*i)->mpi_rank),sizeof(int));
      octaveproc->send(&((*i)->mpi_pid),sizeof(int));
    }
  }

  regist();
  
  map_mpi_comm.insert(std::make_pair(0,MPI_COMM_WORLD));
  map_mpi_comm.insert(std::make_pair(1,MPI_COMM_SELF));

  loop(log,octaveproc);

  //Cancel All unexecuted MPI-function calls

  //Cancels a communication request
  data_conv_map<int,req_bin *>::iterator it;
  for(it=map_mpi_request.begin();it!=map_mpi_request.end();it++){
    MPI_Cancel(&it->second->request);
  }

  MPI_Finalize();

  log->writelog("MPI_Finalize called");
  delete log;

  return EXIT_SUCCESS;

};
