//Octave include files

#include <octave/oct.h>
#include <variables.h>
#include <oct-map.h>

//Socket programming include files
#include <sys/socket.h>
#include <sys/types.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

struct domain{
  static int get_domain(const std::string domain){
    if( (domain==std::string("PF_UNIX")) || (domain==std::string("AF_UNIX")) ){
      return  PF_UNIX;
    }
    if( (domain==std::string("PF_INET")) || (domain==std::string("AF_INET")) ){
      return PF_INET;
    }
    return -1;
  };
};

bool str_to_addr(const std::string host_addr,struct in_addr *addr){
  //When address format is "127.0.0.1"
  if(inet_aton(host_addr.c_str(),addr) != 0)
    return true;
  
  //When address format is "localhost"
  struct hostent *host;
  host=gethostbyname(host_addr.c_str());

  if( host != NULL )
    {
      struct in_addr *aptr;
      aptr=(struct in_addr *)*host->h_addr_list;
      memcpy(addr,aptr,sizeof(struct in_addr));
      return true;
    }
  
  return false;
}

//  bool str_to_port(std::string port,std::string tcp_udp,int &portno){
//    struct servent *se;

//    //When port format is "http" and tcp_ucp format is "tcp" or "udp"
//    se = getservbyname(port.c_str(),tcp_udp.c_str());
//    if( se != NULL){
//      portno= se->s_port;
//      return true;
//    }
  
//    return false;
//  };

DEFUN_DLD (connect, args,nargout,
 "-*- texinfo -*-\n\
@deftypefn {Dynamic-Loadable Function} {[@var{bool},@var{msg}] =} connect (@var{sockfd}, @var{host}, @var{port})\n\
\n\
Initiate a connection on a socket.\n\
\n\
The value @var{sockfd} must refer to a socket.\n\
\n\
The value @var{host} is either a host name, or an IPv4 address in standard dot notation.\n\
\n\
The value @var{port} is port number for the service.\n\
\n\n\
The function @code{connect} returns @var{bool} and @var{msg},\n\
If the connection or binding succeeds, @var{bool} is set zero and @var{msg} is set \"SUCCESS\".\n\
On error, @var{bool} is set -1, and @var{msg} returned error message\n\
For example,\n\
\n\
@example\n\
@group\n\
[fd msg] = socket ( \"AF_INET\" , \"SOCK_STREAM\" , 0 );\n\
[retval msg] = connect ( fd , \"localhost\" , 80 );\n\
@end group\n\
@end example\n\
@end deftypefn")
{
  int nargin=args.length();
  octave_value_list retval;
  
  if(nargin != 3){
    print_usage("connect");
    return retval;
  }
  
  Octave_map map;
  std::string host;
  int portno;

  map   =args(0).map_value();
  host  =args(1).string_value();
  portno=htons(args(2).int_value());

  if(error_state) return retval=octave_value(-1);
  
  struct sockaddr_in server;
  struct in_addr addr;

  if(! str_to_addr(host,&addr))
    {
      std::string err_msg=std::strerror(errno);
      error("%s",err_msg.c_str());
      return retval=octave_value(-1);
    }
  
  memset(&server,0,sizeof(sockaddr_in));

  int domain=domain::get_domain(map["domain"](0).string_value());
  if(domain < 0)
    {
      error("unsupported domain %s",map["domain"](0).string_value().c_str());
      return retval=octave_value(-1);
    }

  server.sin_family=domain;
  
  server.sin_port=portno;
  server.sin_addr=addr;
  
  int fd=map["fd"](0).int_value();

  int ret = connect(fd,(struct sockaddr *)&server,sizeof(server));
  if(ret == -1) 
    {
      std::string msg = std::strerror (errno);
      retval(1)=msg;
    }
  else
    {
      retval(1)=octave_value("SUCCESS");
    }

  retval(0)=ret;

  return retval;
}

