//Octave include files
#include <octave/oct.h>
#include <variables.h>
#include <oct-map.h>

//Socket programming include files
#include <sys/socket.h>
#include <sys/types.h>

static int get_domain(const std::string domain){
  if( (domain==std::string("PF_UNIX")) || (domain==std::string("AF_UNIX")) ){
    return  PF_UNIX;
  }
  if( (domain==std::string("PF_INET")) || (domain==std::string("AF_INET")) ){
    return PF_INET;
  }
  return -1;
};

static int get_type(const std::string type){
  if(type==std::string("SOCK_STREAM")){
    return SOCK_STREAM;
  }
  if(type==std::string("SOCK_DGRAM")){
    return SOCK_DGRAM;
  }
  return -1;
};

DEFUN_DLD (socket, args, nargout,
 "-*- texinfo -*-\n\
@deftypefn {Dynamic-Loadable Function} {[@var{fd} @var{msg}] =} socket (@var{domain}, @var{type} ,@var{protocol})\n\
create an endpoint for communication.\n\
\n\
The possible values @var{domain} may be\n\
@table @code\n\
@item \"PF_UNIX\",\"AF_UNIX\"\n\
Local communication.\
@item \"PF_INET\",\"AF_INET\"\n\
IPv4 Internet protocols.\
\n\
@end table\n\
\n\
The possible values @var{type} may be\n\
@table @code\n\
@item \"SOCK_STREAM\"\n\
Provides sequenced, reliable, two-way, connection-based byte streams.\n\
@item \"SOCK_DGRAM\"\n\
Supports datagrams.\n\
\n\
@end table\n\
\n\
The value @var{protocol} specifies a particular protocol to be used with the socket.\n\
\n\
For example,\n\
\n\
@example\n\
@group\n\
[fd msg] = socket ( \"AF_INET\" , \"SOCK_STREAM\" , 0 );\n\
@end group\n\
@end example\n\
@end deftypefn")
{
  int nargin=args.length();
  octave_value_list retval;

  if(nargin != 3){
    print_usage("socket");
    return retval;
  }
  
  std::string domain_str;
  std::string type_str;  
  int protocol;

  domain_str=args(0).string_value();
  type_str  =args(1).string_value();
  protocol  =args(2).int_value();
  if(error_state) return retval;
  
  int domain=get_domain(domain_str);
  if(domain < 0){
    error("unsupported domain %s",domain_str.c_str());
    return retval;
  }

  int type=get_type(type_str);
  if(type < 0){
    error("unsupported type %s",type_str.c_str());
    return retval(0)=octave_value(-1);
  }
  
  int soc=socket(domain,type,protocol);
  
  if(soc < 0){
    std::string msg = std::strerror (errno);
    retval(1)=msg;
  }
  else
    retval(1)=octave_value("SUCCESS");

  Octave_map m;
  
  m ["fd"](0)     = soc;
  m ["domain"](0) = domain_str;
  m ["type"](0)   = type_str;
  m ["proto"](0)  = protocol;
    
  retval(0)=m;

  return retval;
}



