#include <unistd.h>
#include <stdio.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <map>
#include <list>
#include <setjmp.h>
#include <sstream>
#include <signal.h>

#include "connect_host.h"
#include "fifo_host.h"
#include "log.h"
#include "fileplace.h"

#define BUFFSIZE 5120

string create_startup_script(const string &fileplace_prefix){

  int pid=getpid();
  char buff[512];
  char hostname[512];
  gethostname(hostname,sizeof(hostname));
  snprintf(buff,sizeof(buff),"r_init_%s_%d",hostname,pid);
  string script_path=Fileplace::getpath()+string(buff);
  FILE *fp=fopen(script_path.c_str(),"w");
  fprintf(fp,"warn_future_time_stamp=0;\n");
  fprintf(fp,"fileplace_prefix=\"%s\";\n",fileplace_prefix.c_str());
  fprintf(fp,"remote_command_server;\n");
  fclose(fp);
  return script_path;

};

connect_host *server;
fifo_host *octave;
Log *logfile=NULL;

void setlogfile(void){
  char hostname[512];
  gethostname(hostname,sizeof(hostname));
  string filename=Fileplace::getpath()+hostname+string("_wrap.log");
  logfile=new Log(filename,O_CREAT|O_WRONLY|O_APPEND);
};

pid_t octave_pid;
pid_t child;

bool octave_is_alive;

void sig_child(int sig){
  int stat;
  wait(&stat);
  if(logfile)
    logfile->writelog("INFO:confirm Octave process quit");
  octave_is_alive=false;

  signal(SIGCHLD,SIG_IGN);
  kill(child,SIGUSR1);
  
  wait(&stat);
  if(logfile)
    logfile->writelog("INFO:confirm Child Wrapper process quit");
};

int child_fdmw;
int child_fdmr;
int child_fdme;

void siguser(int sig){
  close(child_fdmw);
  close(child_fdmr);
  close(child_fdme);
  server->stop();
  delete server;
  logfile->writelog("INFO:listener process quit");
  exit(EXIT_SUCCESS);
};

int s_loop(int fdmw,int fdmr,int fdme,std::string startup_host,int linked_mpi_rank){
  signal(SIGCHLD,sig_child);

  int  nread=0;
  char buff[BUFFSIZE];
  logfile->writelog("INFO:enter loop");

  if((child=fork())==0){
    child_fdmw=fdmw;
    child_fdmr=fdmr;
    child_fdme=fdme;

    server=new connect_host(startup_host,23457);
    signal(SIGUSR1,siguser);

    server->start();

    char hostname[512];
    gethostname(hostname,sizeof(hostname));
    string octave_fifo1=Fileplace::getpath()+hostname+"wrapfifo1";
    string octave_fifo2=Fileplace::getpath()+hostname+"wrapfifo2";

    octave=new fifo_host(octave_fifo1,octave_fifo2,2);
    octave->start();

    pid_t linked_octave_pid;


    //send MPI Rank to central process
    //(for synchronization toword mpi_init.oct)
    server->send(&linked_mpi_rank,sizeof(int));

    //recv process ID from Octave process (in remote_command_server.oct)
    octave->recv(&linked_octave_pid,sizeof(pid_t));

    //send Octave process ID to central process
    server->send(&octave_pid,sizeof(pid_t));

    //send wrapper(own) process ID to central process
    pid_t my_pid;
    my_pid=getpid();
    server->send(&my_pid,sizeof(pid_t));

    //server->send(&my_pid,sizeof(pid_t));

    logfile->writelog("INFO:dup finish");

    while(true){
      //      if(server->scanstream(server->getremotehostname(0),-1)==true){
      std::string remote_hostname;
      pid_t remote_pid;
      
      if(server->scan_any_stream(remote_hostname,remote_pid,-1) == 1){
	nread=server->recv(buff,sizeof(buff));
	octave->send(buff,nread);
	logfile->writelog("INFO:stdin[%s]",buff,nread);
      }
    }
    assert(false);
    return 1;
  }
  else{
    struct pollfd pfd[2];
    pfd[0].fd=fdme;
    pfd[0].events=POLLIN;
    pfd[1].fd=fdmr;
    pfd[1].events=POLLIN;

    while(octave_is_alive){
      int fdnum=poll(pfd,2,-1);
      if(fdnum>0){
        for(int i=0;i<2;i++){
          if(pfd[i].revents==POLLIN){
            memset(buff,'\0',sizeof(buff));
            nread=read(pfd[i].fd, buff, BUFFSIZE);

            //when child process finish and parent still this section,
	    //assertion may occure.
	    //(because SIGCHLD arrive after "read" call
	    //so I comment out this assertion.
	    //assert(nread!=-1);

            if(i==0){
	      logfile->writelog("INFO:stderr[%s]",buff);
	      write(STDERR_FILENO,buff,nread);
	    }
            else{
	      logfile->writelog("INFO:stdout[%s]",buff);
	    }
          }
        }
      }
    }
    return 1;
  }
}

int
ptym_open(int mode,char *pts_name,int len)
{
  int             fdm;
  char    *ptr1, *ptr2;
  
  if(len<9)
    return -1;
  memset(pts_name,'\0',len);

  strncpy(pts_name,"/dev/ptyXY",strlen("/dev/ptyXY"));
  /* array index: 0123456789 (for references in following code) */
  //for (ptr1 = "pqrstuvwxyzPQRST"; *ptr1 != 0; ptr1++) {
  for (ptr1 = "qrstuvwxyzPQRST"; *ptr1 != 0; ptr1++) {
    pts_name[8] = *ptr1;
    for (ptr2 = "0123456789abcdef"; *ptr2 != 0; ptr2++) {
      pts_name[9] = *ptr2;
      
      /* try to open master */
      if ( (fdm = open(pts_name,mode)) < 0) {
	if (errno == ENOENT)    /* different from EIO */
	  return(-1);                     /* out of pty devices */
	else
	  continue;                       /* try next pty device */
      }
      else{
	pts_name[5] = 't';      /* change "pty" to "tty" */
	return(fdm);            /* got it, return fd of master */
      }
    }
  }
  return(-1);             /* out of pty devices */
}

int main(int argc,char *argv[]){
  string startup_host;

  //NEED FIX
  //the name "fileplace_prefix" be changed to "linked_mpi_rank"
  string fileplace_prefix;
  string path_to_octave;
 
  int   c;
  while((c=getopt(argc,argv,"s:r:h:p:"))!=EOF){
    switch (c) {
    case 's':
      assert(chdir(optarg)==0);
      //logfile->writelog("INFO:current directory = %s",optarg);
      break;
    case 'r':
      fileplace_prefix=string(optarg);
      //logfile->writelog("INFO:prefix of file-save path = %s",optarg);
      break;
    case 'h':
      startup_host=string(optarg);
      //logfile->writelog("INFO:the hostname to connect = %s",optarg);
      break;
    case 'p':
      path_to_octave=string(optarg);
      //logfile->writelog("INFO:the hostname to connect = %s",optarg);
      break;
    default:
      fprintf(stderr,"unknown option\n");
      exit(0);
    }
  }

  Fileplace::set_prefix(fileplace_prefix);

  string script_path=create_startup_script(fileplace_prefix);

  setlogfile();

  char fdmw_s_name[25];
  char fdmr_s_name[25];
  char fdme_s_name[25];

  int fdmw= ptym_open(O_WRONLY,fdmw_s_name,25);
  int fdmr= ptym_open(O_RDONLY,fdmr_s_name,25);
  int fdme= ptym_open(O_RDONLY,fdme_s_name,25);

  char hostname[512];
  gethostname(hostname,512);

  assert(fdmw!=-1);
  assert(fdmr!=-1);
  assert(fdme!=-1);
  
  octave_pid=fork();
  octave_is_alive=true;
  if(octave_pid==0){ /* child */
    close(fdmw);
    close(fdmr);
    close(fdme);
    setsid();

    int fdsr=open(fdmw_s_name,O_RDONLY);
    int fdsw=open(fdmr_s_name,O_WRONLY);
    int fdse=open(fdme_s_name,O_WRONLY);

    if(fdsr==-1){
      perror("");
      fprintf(stderr,"WORN pty_r in %s ,ptyname=%s\n",hostname,fdmw_s_name);
    };

    if(fdsw==-1){
      perror("");
      fprintf(stderr,"WORN pty_w in %s ,ptyname=%s\n",hostname,fdmr_s_name);
    };
    
    if(fdse==-1){
      perror("");
      fprintf(stderr,"WORN pty_e in %s ,ptyname=%s\n",hostname,fdme_s_name);
    };

    dup2(fdsr,STDIN_FILENO);
    dup2(fdsw,STDOUT_FILENO);
    dup2(fdse,STDERR_FILENO);

    execlp(path_to_octave.c_str(),
           path_to_octave.c_str(),
	   "-q",
	   "--no-history",
	   script_path.c_str(),
	   NULL);

    assert(false);
  }
  else{
    int linked_mpi_rank_int = atoi(fileplace_prefix.c_str());
    s_loop(fdmw,fdmr,fdme,startup_host,linked_mpi_rank_int);
  }

  close(fdmw);
  close(fdmr);
  close(fdme);

  delete logfile;
  remove(script_path.c_str());

  return EXIT_SUCCESS;
}







